unit ComPortUnit;

interface

uses
  Windows, Messages, SysUtils, Variants, Classes, Graphics, Controls, Forms,
  Dialogs, StdCtrls, SafeListsUnit, DateUtils;


type
  TComPort = class(TThread)
  private
    { Private declarations }
  protected
    procedure Execute; override;
    procedure AddToError(str: string);
    procedure AddToInput(str: string);
    procedure AddToDebug(str: string);
    function  GetNextOutput:string;
  public
    doExit: bool;
    idleState: bool;
    myParent: TForm;
    com_port: string;
    baud_rate: integer;
    constructor Create(CreateSuspended: Boolean; pw: TForm; cp: string; BaudRate: integer);
  end;

implementation

uses Main;


{ Important: Methods and properties of objects in VCL or CLX can only be used
  in a method called using Synchronize, for example,

      Synchronize(UpdateCaption);

  and UpdateCaption could look like,

    procedure TComPort.UpdateCaption;
    begin
      Form1.Caption := 'Updated in a thread';
    end; }

{ TComPort }

procedure TComPort.AddToError(str: string);
begin

   (myParent as TMainForm).Error.GetAccess;
   try
     (myParent as TMainForm).Error.Add(str);
   finally
     (myParent as TMainForm).Error.ReleaseAccess;
   end;

end;

procedure TComPort.AddToInput(str: string);
begin

   if Copy(str,1,5) = '$GPS:' then begin
       (myParent as TMainForm).GPS_mon := Copy(str,6,Length(str));
   end else
   if Copy(str,1,5) = '$ADC:' then begin
       (myParent as TMainForm).ADC_mon := Copy(str,6,Length(str));
   end else
   if Copy(str,1,5) = '$PPM:' then begin
       (myParent as TMainForm).PPM_mon := Copy(str,6,Length(str));
   end else
   if Copy(str,1,5) = '$MAG:' then begin
       (myParent as TMainForm).MAG_mon := Copy(str,6,Length(str));
   end else
   if Copy(str,1,5) = '$EKF:' then begin
       (myParent as TMainForm).EKF_mon := Copy(str,6,Length(str));
   end else
   if Copy(str,1,6) = '$GYRO:' then begin
       (myParent as TMainForm).GYRO_mon := Copy(str,7,Length(str));
   end else
   if Copy(str,1,7) = '$ACCEL:' then begin
       (myParent as TMainForm).ACCEL_mon := Copy(str,8,Length(str));
   end else
   if Copy(str,1,6) = '$BARO:' then begin
       (myParent as TMainForm).BARO_mon := Copy(str,7,Length(str));
   end else
   if Copy(str,1,6) = '$STAB:' then begin
       (myParent as TMainForm).STAB_mon := Copy(str,7,Length(str));
   end
   else begin
     (myParent as TMainForm).Input.GetAccess;
     try
       (myParent as TMainForm).Input.Add(str);
     finally
       (myParent as TMainForm).Input.ReleaseAccess;
     end;

   end;

end;

{
procedure TComPort.AddToMonitor(str: string);
begin

   (myParent as TMainForm).Monitor.GetAccess;
   try
     (myParent as TMainForm).Monitor.Add(str);
   finally
     (myParent as TMainForm).Monitor.ReleaseAccess;
   end;

end;
}

procedure TComPort.AddToDebug(str: string);
begin

   (myParent as TMainForm).DebugMon.GetAccess;
   try
     (myParent as TMainForm).DebugMon.Add(str);
   finally
     (myParent as TMainForm).DebugMon.ReleaseAccess;
   end;

end;

function TComPort.GetNextOutput: string;
begin

   Result:='';
   (myParent as TMainForm).Output.GetAccess;
   try
     if (myParent as TMainForm).Output.Count>0 then begin
        Result:=(myParent as TMainForm).Output[0];
        (myParent as TMainForm).Output.Delete(0);
     end;
   finally
     (myParent as TMainForm).Output.ReleaseAccess;
   end;
 
end;


constructor TComPort.Create(CreateSuspended: Boolean; pw: TForm; cp: string; BaudRate: integer);
begin
  inherited Create(CreateSuspended);
  myParent:=pw;
  com_port:=cp;
  Baud_Rate:=BaudRate;


end;

procedure TComPort.Execute;
var
  hComm: THandle;
  dcbInfo:Tdcb;
  errorFlag:Boolean;
  buf: string;
  readCount,writeCount: DWORD;
  commPortError:DWORD;
  commPortStatus:TComStat;
  resultCode: string;
  sendCode: string;
  i: integer;
  ts: TDateTime;
begin
  doExit:=false;
  idleState:=false;
try
  hComm:= CreateFile(PChar(com_port),GENERIC_READ or GENERIC_WRITE  ,0,nil,OPEN_EXISTING,0,0);
  if (hComm=INVALID_HANDLE_VALUE) then begin
      AddToError('Cannot open '+com_port+' port, error: '+IntToStr(GetLastError()));
      PostMessage(MyParent.Handle,ERROR_MSG,1,0);
      exit;
  end;
  errorFlag:=GetCommState(hComm,dcbInfo);
  if (NOT(errorFlag)) then
     begin
         CloseHandle(hComm);
           AddToError('Connot read '+com_port+' params');
           PostMessage(MyParent.Handle,ERROR_MSG,1,0);
           exit;
     end;

     // setup COMM port settings
   dcbInfo.BaudRate:=Baud_Rate;
   dcbInfo.ByteSize:=8;
   dcbInfo.Parity:=NOPARITY;
   dcbInfo.StopBits:=ONESTOPBIT;
//   dcbInfo.EofChar:=#13;
   // set COMM port state
   errorFlag:=SetCommState(hcomm,dcbInfo);
   // bail on failure
   if (NOT(errorFlag)) then
       begin
           CloseHandle(hcomm);
           AddToError('Connot set '+com_port+' params');
           PostMessage(MyParent.Handle,ERROR_MSG,1,0);
           exit;
       end;



   PostMessage(MyParent.Handle,CONNECT_MSG,0,0);
   ts := now;
   while not DoExit do begin
     resultCode:='';
     while true do begin
       ClearCommError(hcomm,commPortError,@commPortStatus);
       if (commPortStatus.cbInQue<>0) then begin
         ts := now;
         SetLength(buf,commPortStatus.cbInQue);
         ReadFile(hcomm,buf[1],   commPortStatus.cbInQue,readCount,nil);
         SetLength(buf,readCount);
         resultCode:=resultCode+buf;
         while Pos(#13#10,resultCode)>0 do begin
           i:=  Pos(#13#10,resultCode);
           AddToInput(Copy(resultCode,1,i-1));
           AddToDebug(Copy(resultCode,1,i-1));
           PostMessage(MyParent.Handle,INPUT_MSG,0,0);
           PostMessage(MyParent.Handle,DEBUG_MON_MSG,0,0);
           Delete(resultCode,1,i+1);
         end;
         
       end else begin
         if Length(resultCode) = 2 then begin
           if resultCode='=>' then begin
             if MilliSecondsBetween(Now,ts) > 100 then begin
                AddToInput(resultCode);
                AddToDebug(resultCode);
                resultCode := '';
                PostMessage(MyParent.Handle,INPUT_MSG,0,0);
                PostMessage(MyParent.Handle,DEBUG_MON_MSG,0,0);
             end;
           end;
         end;

         //       
         sendCode:=GetNextOutput;
         if Length(sendCode) > 0 then begin
            WriteFile(hcomm,sendCode[1],length(sendCode),writeCount,nil);
            AddToDebug(trim(sendCode));
            PostMessage(MyParent.Handle,DEBUG_MON_MSG,0,0);
         end;

       end;
       if doExit then begin
          resultCode:='';
          break;
       end;
       Sleep(20);
     end;
   end;


   CloseHandle(hComm);
   idleState:=true;
finally
   PostMessage(MyParent.Handle,COMPORT_DISCONNECT,0,0);
//   self.Free;
end;


end;


end.
 